__author__ = "Mislav Novakovic <mislav.novakovic@sartura.hr>"
__copyright__ = "Copyright 2017, Deutsche Telekom AG"
__license__ = "BSD 3-Clause"

import yang as ly

ctx = None
try:
    ctx = ly.Context("/etc/sysrepo2/yang")
except Exception as e:
    print(e)
    errors = ly.get_ly_errors(ctx)
    for err in errors:
        print("err: %d" % err.err())
        print("vecode: %d" % err.vecode())
        print("errmsg: "+err.errmsg())
        print("errpath:"+err.errpath())
        print("errapptag:"+err.errapptag())

try:
    ctx = ly.Context("/etc/sysrepo/yang")
except Exception as e:
    print(e)

folders = ctx.get_searchdirs()
for folder in folders:
    print(folder)
print("\n")

module = ctx.get_module("ietf-interfaces")
if module is not None:
    print(module.name())
else:
    module = ctx.load_module("ietf-interfaces")
    if module is not None:
        print(module.name())

modules = ctx.get_module_iter()
for module in modules:
    print("module: " + module.name() + " prefix: " + module.prefix() + " type: " + str(module.type()))



__author__ = "Mislav Novakovic <mislav.novakovic@sartura.hr>"
__copyright__ = "Copyright 2017, Deutsche Telekom AG"
__license__ = "BSD 3-Clause"

import yang as ly
import sys

ctx = None
try:
    ctx = ly.Context("/etc/sysrepo/yang")

    module = ctx.load_module("turing-machine", None)
    if module is None:
        print("module not loaded")
        sys.exit()

    node = None
    try:
        if node is None : node = ctx.parse_data_path("/etc/sysrepo/data/turing-machine.startup", ly.LYD_LYB, ly.LYD_OPT_CONFIG)
    except Exception as e:
        print(e)
    try:
        if node is None : node = ctx.parse_data_path("/etc/sysrepo/data/turing-machine.startup", ly.LYD_XML, ly.LYD_OPT_CONFIG)
    except Exception as e:
        print(e)
    try:
        if node is None : node = ctx.parse_data_path("/etc/sysrepo/data/turing-machine.startup", ly.LYD_JSON, ly.LYD_OPT_CONFIG)
    except Exception as e:
        print(e)

    if node is None:
        sys.exit()

    node_set = node.find_path("/turing-machine:turing-machine/transition-function/delta[label='left summand']/*")
    if node_set is None:
        print("could not find data for xpath")
        sys.exit()

    for data_set in node_set.data():
        schema = data_set.schema()
        print("name: " + schema.name() + " type: " + str(schema.nodetype()) + " path: " + data_set.path())

except Exception as e:
    print(e)
    errors = ly.get_ly_errors(ctx)
    for err in errors:
        print("err: %d" % err.err())
        print("vecode: %d" % err.vecode())
        print("errmsg: " + err.errmsg())
        print("errpath:" + err.errpath())
        print("errapptag:" + err.errapptag())



__author__ = "Mislav Novakovic <mislav.novakovic@sartura.hr>"
__copyright__ = "Copyright 2017, Deutsche Telekom AG"
__license__ = "BSD 3-Clause"

import yang as ly
import sys

try:
    ctx = ly.Context("/etc/sysrepo/yang")
except Exception as e:
    print(e)
    sys.exit()

ctx.load_module("iana-if-type", None)
ctx.load_module("ietf-inet-types", None)
ctx.load_module("ietf-yang-types", None)
ctx.load_module("ietf-interfaces", None)
ctx.load_module("ietf-ip", None)

node = None
try:
    if node is None : node = ctx.parse_data_path("/etc/sysrepo/data/ietf-interfaces.startup", ly.LYD_LYB, ly.LYD_OPT_CONFIG)
except Exception as e:
    print(e)
try:
    if node is None : node = ctx.parse_data_path("/etc/sysrepo/data/ietf-interfaces.startup", ly.LYD_XML, ly.LYD_OPT_CONFIG)
except Exception as e:
    print(e)
try:
    if node is None : node = ctx.parse_data_path("/etc/sysrepo/data/ietf-interfaces.startup", ly.LYD_JSON, ly.LYD_OPT_CONFIG)
except Exception as e:
    print(e)

if node is None:
    sys.exit()

node_set = node.find_path("/ietf-interfaces:interfaces//*")
if node_set is None:
    print("could not find data for xpath")
    sys.exit()

for data_set in node_set.data():
    schema = data_set.schema()
    print("name: " + schema.name() + " type: " + str(schema.nodetype()) + " path: " + data_set.path())
    if (schema.nodetype() == ly.LYS_LIST):
        if_list = schema.subtype()
        print("list's key names are: " + if_list.keys_str())



#!/usr/bin/env python
from __future__ import print_function

__author__ = "Matija Amidzic <matija.amidzic@sartura.hr>"
__copyright__ = "Copyright 2018, Deutsche Telekom AG"
__license__ = "BSD 3-Clause"

# This source code is licensed under BSD 3-Clause License (the "License").
# You may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://opensource.org/licenses/BSD-3-Clause

import yang as ly
import unittest
import sys

import config

lys_module_a = \
"<?xml version=\"1.0\" encoding=\"UTF-8\"?>           \
<module name=\"a\"                                    \
        xmlns=\"urn:ietf:params:xml:ns:yang:yin:1\"   \
        xmlns:md=\"urn:ietf:params:xml:ns:yang:ietf-yang-metadata\"\
        xmlns:a=\"urn:a\">                            \
  <namespace uri=\"urn:a\"/>                          \
  <prefix value=\"a_mod\"/>                           \
  <include module=\"asub\"/>                          \
  <include module=\"atop\"/>                          \
  <import module=\"ietf-yang-metadata\">              \
    <prefix value=\"md\"/>                            \
  </import>                                           \
  <feature name=\"foo\"/>                             \
  <grouping name=\"gg\">                              \
    <leaf name=\"bar-gggg\">                          \
      <type name=\"string\"/>                         \
    </leaf>                                           \
  </grouping>                                         \
  <md:annotation name=\"test\">                       \
    <type name=\"string\"/>                           \
  </md:annotation>                                    \
  <container name=\"x\">                              \
    <leaf name=\"bar-leaf\">                          \
      <type name=\"string\"/>                         \
    </leaf>                                           \
    <uses name=\"gg\">                                \
    </uses>                                           \
    <leaf name=\"baz\">                               \
      <type name=\"string\"/>                         \
    </leaf>                                           \
    <leaf name=\"bubba\">                             \
      <type name=\"string\"/>                         \
    </leaf>                                           \
    <leaf name=\"number32\">                          \
      <type name=\"int32\"/>                          \
    </leaf>                                           \
    <leaf name=\"number64\">                          \
      <type name=\"int64\"/>                          \
    </leaf>                                           \
    <leaf name=\"def-leaf\">                          \
      <type name=\"string\"/>                         \
      <default value=\"def\"/>                        \
    </leaf>                                           \
    <anydata name=\"any-data\"/>                      \
  </container>                                        \
  <leaf name=\"y\"><type name=\"string\"/></leaf>     \
  <anyxml name=\"any\"/>                              \
  <augment target-node=\"/x\">                        \
    <container name=\"bar-y\"/>                       \
  </augment>                                          \
  <rpc name=\"bar-rpc\">                              \
  </rpc>                                              \
  <rpc name=\"foo-rpc\">                              \
  </rpc>                                              \
  <rpc name=\"rpc1\">                                 \
    <input>                                           \
      <leaf name=\"input-leaf1\">                     \
        <type name=\"string\"/>                       \
      </leaf>                                         \
      <container name=\"x\">                          \
        <leaf name=\"input-leaf2\">                   \
          <type name=\"string\"/>                     \
        </leaf>                                       \
      </container>                                    \
    </input>                                          \
    <output>                                          \
      <leaf name=\"output-leaf1\">                    \
        <type name=\"string\"/>                       \
      </leaf>                                         \
      <leaf name=\"output-leaf2\">                    \
        <type name=\"string\"/>                       \
      </leaf>                                         \
      <container name=\"rpc-container\">              \
        <leaf name=\"output-leaf3\">                  \
          <type name=\"string\"/>                     \
        </leaf>                                       \
      </container>                                    \
    </output>                                         \
  </rpc>                                              \
  <list name=\"l\">                                   \
    <key value=\"key1 key2\"/>                        \
    <leaf name=\"key1\">                              \
      <type name=\"uint8\"/>                          \
    </leaf>                                           \
    <leaf name=\"key2\">                              \
      <type name=\"uint8\"/>                          \
    </leaf>                                           \
    <leaf name=\"value\">                             \
      <type name=\"string\"/>                         \
    </leaf>                                           \
  </list>                                             \
</module>                                             \
"

a_data_xml = "\
<x xmlns=\"urn:a\">\n\
  <bubba>test</bubba>\n\
  </x>\n"

result_xml = "<x xmlns=\"urn:a\"><bubba>test</bubba></x>"

result_xml_format ="\
<x xmlns=\"urn:a\">\n\
  <bubba>test</bubba>\n\
</x>\n\
"

result_json = "\
{\n\
  \"a:x\": {\n\
    \"bubba\": \"test\"\n\
  }\n\
}\n\
"

class UnexpectedError(Exception):
    """Exception raised for errors that are not expected.

    Attributes:
        message -- explanation of the error
    """

    def __init__(self, message):
        self.message = message

class TestUM(unittest.TestCase):
    def test_ly_ctx_parse_data_mem(self):
        yang_folder = config.TESTS_DIR + "/api/files"
        yin_file = config.TESTS_DIR + "/api/files/a.yin"

        try:
            # Setup
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)
            ctx.parse_module_path(yin_file, ly.LYS_IN_YIN)

            # Tests
            root = ctx.parse_data_mem(a_data_xml, ly.LYD_XML, ly.LYD_OPT_NOSIBLINGS | ly.LYD_OPT_STRICT)
            self.assertIsNotNone(root)
            self.assertEqual("x", root.schema().name())

        except Exception as e:
            self.fail(e)

    def test_ly_ctx_parse_data_fd(self):
        yang_folder = config.TESTS_DIR + "/api/files"
        yin_file = config.TESTS_DIR + "/api/files/a.yin"
        config_file = config.TESTS_DIR + "/api/files/a.xml"
        try:
            # Setup
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)
            ctx.parse_module_path(yin_file, ly.LYS_IN_YIN)

            # Tests
            f = open(config_file, 'r')
            fd = f.fileno()
            root = ctx.parse_data_fd(fd, ly.LYD_XML, ly.LYD_OPT_NOSIBLINGS | ly.LYD_OPT_STRICT)
            self.assertIsNotNone(root)
            self.assertEqual("x", root.schema().name())

        except Exception as e:
            self.fail(e)

        finally:
            f.close()

    def test_ly_ctx_parse_data_path(self):
        yang_folder = config.TESTS_DIR + "/api/files"
        yin_file = config.TESTS_DIR + "/api/files/a.yin"
        config_file = config.TESTS_DIR + "/api/files/a.xml"
        module_name = "a"
        schema_name = "x"
        try:
            # Setup
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)
            module = ctx.parse_module_path(yin_file, ly.LYS_IN_YIN)
            self.assertIsNotNone(module)
            self.assertEqual(module_name, module.name(), "Module names don't match")

            # Tests
            root = ctx.parse_data_path(config_file, ly.LYD_XML, ly.LYD_OPT_CONFIG | ly.LYD_OPT_STRICT)
            self.assertIsNotNone(root)
            self.assertEqual(schema_name, root.schema().name())

        except Exception as e:
            self.fail(e)

    def test_ly_ctx_parse_data_path_invalid(self):
        yang_folder = config.TESTS_DIR + "/api/files"

        try:
            # Setup
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)

            # Tests
            root = ctx.parse_data_path("INVALID_PATH", ly.LYD_XML, ly.LYD_OPT_CONFIG | ly.LYD_OPT_STRICT)
            raise UnexpectedError("exception not thrown")

        except UnexpectedError as e:
            self.fail(e)

        except RuntimeError as e:
            return

        except Exception as e:
            self.fail(e)

    def test_ly_data_node(self):
        yang_folder = config.TESTS_DIR + "/api/files"
        config_file = config.TESTS_DIR + "/api/files/a.xml"
        try:
            # Setup
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)
            ctx.parse_module_mem(lys_module_a, ly.LYS_IN_YIN)
            root = ctx.parse_data_path(config_file, ly.LYD_XML, ly.LYD_OPT_CONFIG | ly.LYD_OPT_STRICT)
            self.assertIsNotNone(root)

            # Tests
            new_node = ly.Data_Node(root, root.schema().module(), "number32", "100")
            self.assertIsNotNone(new_node)

        except Exception as e:
            self.fail(e)

    def test_ly_data_node_new_path(self):
        yang_folder = config.TESTS_DIR + "/api/files"
        config_file = config.TESTS_DIR + "/api/files/a.xml"
        try:
            # Setup
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)
            ctx.parse_module_mem(lys_module_a, ly.LYS_IN_YIN)
            ctx.parse_data_path(config_file, ly.LYD_XML, ly.LYD_OPT_CONFIG | ly.LYD_OPT_STRICT)
            mod = ctx.get_module("a", None, 1)
            self.assertIsNotNone(mod)

            # Tests
            root = ly.Data_Node(ctx, "/a:x/bar-gggg", "a", 0, 0)
            self.assertIsNotNone(root)
            self.assertEqual("x", root.schema().name())
            self.assertEqual("bar-gggg", root.child().schema().name())

            node = root.new_path(ctx, "def-leaf", "def", 0, ly.LYD_PATH_OPT_DFLT)
            self.assertIsNotNone(node)
            self.assertEqual("def-leaf", node.schema().name())
            self.assertEqual(1, node.dflt())

            node = root.new_path(ctx, "def-leaf", "def", 0, 0)
            self.assertIsNotNone(node)
            self.assertEqual("def-leaf", node.schema().name())
            self.assertEqual(0, node.dflt())

            node = root.new_path(ctx, "bubba", "b", 0, 0)
            self.assertIsNotNone(node)
            self.assertEqual("bubba", node.schema().name())

            node = root.new_path(ctx, "/a:x/number32", "3", 0, 0)
            self.assertIsNotNone(node)
            self.assertEqual("number32", node.schema().name())

            node = root.new_path(ctx, "/a:l[key1='1'][key2='2']/value", None, 0, 0)
            self.assertIsNotNone(node)
            self.assertEqual("l", node.schema().name())
            self.assertEqual("key1", node.child().schema().name())
            self.assertEqual("key2", node.child().next().schema().name())
            self.assertEqual("value", node.child().next().next().schema().name())

        except Exception as e:
            self.fail(e)

    def test_ly_data_node_insert(self):
        yang_folder = config.TESTS_DIR + "/api/files"
        config_file = config.TESTS_DIR + "/api/files/a.xml"

        try:
            # Setup
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)
            ctx.parse_module_mem(lys_module_a, ly.LYS_IN_YIN)
            root = ctx.parse_data_path(config_file, ly.LYD_XML, ly.LYD_OPT_CONFIG | ly.LYD_OPT_STRICT)
            self.assertIsNotNone(root)

            # Tests
            new_node = ly.Data_Node(root, root.schema().module(), "number32", "200")
            self.assertIsNotNone(new_node)
            rc = root.insert(new_node)
            self.assertEqual(0, rc)
            self.assertEqual("number32", root.child().prev().schema().name())


        except Exception as e:
            self.fail(e)

    def test_ly_data_node_insert_sibling(self):
        yang_folder = config.TESTS_DIR + "/api/files"
        config_file = config.TESTS_DIR + "/api/files/a.xml"

        try:
            # Setup
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)
            ctx.parse_module_mem(lys_module_a, ly.LYS_IN_YIN)
            root = ctx.parse_data_path(config_file, ly.LYD_XML, ly.LYD_OPT_CONFIG | ly.LYD_OPT_STRICT)
            self.assertIsNotNone(root)

            # Tests
            last = root.prev()
            new_node = ly.Data_Node(None, root.schema().module(), "y", "test")
            self.assertIsNotNone(new_node)
            rc = root.insert_sibling(new_node)
            self.assertEqual(0, rc)
            self.assertNotEqual(last.schema().name(), root.prev().schema().name())
            self.assertEqual("y", root.prev().schema().name())

        except Exception as e:
            self.fail(e)

    def test_ly_data_node_insert_before(self):
        yang_folder = config.TESTS_DIR + "/api/files"
        config_file = config.TESTS_DIR + "/api/files/a.xml"

        try:
            # Setup
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)
            ctx.parse_module_mem(lys_module_a, ly.LYS_IN_YIN)
            root = ctx.parse_data_path(config_file, ly.LYD_XML, ly.LYD_OPT_CONFIG | ly.LYD_OPT_STRICT)
            self.assertIsNotNone(root)

            # Tests
            last = root.prev()
            new_node = ly.Data_Node(None, root.schema().module(), "y", "test")
            self.assertIsNotNone(new_node)
            rc = root.insert_before(new_node)
            self.assertEqual(0, rc)
            self.assertNotEqual(last.schema().name(), root.prev().schema().name())
            self.assertEqual("y", root.prev().schema().name())

        except Exception as e:
            self.fail(e)

    def test_ly_data_node_insert_after(self):
        yang_folder = config.TESTS_DIR + "/api/files"
        config_file = config.TESTS_DIR + "/api/files/a.xml"

        try:
            # Setup
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)
            ctx.parse_module_mem(lys_module_a, ly.LYS_IN_YIN)
            root = ctx.parse_data_path(config_file, ly.LYD_XML, ly.LYD_OPT_CONFIG | ly.LYD_OPT_STRICT)
            self.assertIsNotNone(root)

            # Tests
            last = root.next()
            new_node = ly.Data_Node(None, root.schema().module(), "y", "test")
            self.assertIsNotNone(new_node)
            rc = root.insert_after(new_node)
            self.assertEqual(0, rc)
            self.assertNotEqual(last.schema().name(), root.next().schema().name())
            self.assertEqual("y", root.next().schema().name())

        except Exception as e:
            self.fail(e)

    def test_ly_data_node_schema_sort(self):
        yang_folder = config.TESTS_DIR + "/api/files"
        config_file = config.TESTS_DIR + "/api/files/a.xml"

        try:
            # Setup
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)
            ctx.parse_module_mem(lys_module_a, ly.LYS_IN_YIN)
            ctx.parse_data_path(config_file, ly.LYD_XML, ly.LYD_OPT_CONFIG | ly.LYD_OPT_STRICT)
            mod = ctx.get_module("a", None, 1)
            self.assertIsNotNone(mod)

            # Tests
            root = ly.Data_Node(None, mod, "l")
            self.assertIsNotNone(root)
            node = ly.Data_Node(root, mod, "key1", "1")
            self.assertIsNotNone(node)
            node = ly.Data_Node(root, mod, "key2", "2")
            self.assertIsNotNone(node)

            node = ly.Data_Node(None, mod, "x")
            self.assertIsNotNone(node)
            rc = root.insert_after(node)
            self.assertEqual(0, rc)
            node = root.next()

            node2 = ly.Data_Node(node, mod, "bubba", "a")
            self.assertIsNotNone(node2)
            node2 = ly.Data_Node(node, mod, "bar-gggg", "b")
            self.assertIsNotNone(node2)
            node2 = ly.Data_Node(node, mod, "number64", "64")
            self.assertIsNotNone(node2)
            node2 = ly.Data_Node(node, mod, "number32", "32")
            self.assertIsNotNone(node2)

            rc = root.schema_sort(1)
            self.assertEqual(0, rc)

            root = node
            self.assertEqual("x", root.schema().name())
            self.assertEqual("l", root.next().schema().name())

            self.assertEqual("bar-gggg", root.child().schema().name())
            self.assertEqual("bubba", root.child().next().schema().name())
            self.assertEqual("number32", root.child().next().next().schema().name())
            self.assertEqual("number64", root.child().next().next().next().schema().name())

        except Exception as e:
            self.fail(e)

    def test_ly_data_node_find_path(self):
        yang_folder = config.TESTS_DIR + "/api/files"
        config_file = config.TESTS_DIR + "/api/files/a.xml"
        try:
            # Setup
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)
            ctx.parse_module_mem(lys_module_a, ly.LYS_IN_YIN)
            root = ctx.parse_data_path(config_file, ly.LYD_XML, ly.LYD_OPT_CONFIG | ly.LYD_OPT_STRICT)
            self.assertIsNotNone(root)

            # Tests
            node = root.child()
            self.assertIsNotNone(node)
            set = node.find_path("/a:x/bubba")
            self.assertIsNotNone(set)
            self.assertEqual(1, set.number())

        except Exception as e:
            self.fail(e)

    def test_ly_data_node_find_instance(self):
        yang_folder = config.TESTS_DIR + "/api/files"
        config_file = config.TESTS_DIR + "/api/files/a.xml"
        try:
            # Setup
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)
            ctx.parse_module_mem(lys_module_a, ly.LYS_IN_YIN)
            root = ctx.parse_data_path(config_file, ly.LYD_XML, ly.LYD_OPT_CONFIG | ly.LYD_OPT_STRICT)
            self.assertIsNotNone(root)

            # Tests
            node = root.child()
            self.assertIsNotNone(node)
            set = node.find_instance(node.schema())
            self.assertIsNotNone(set)
            self.assertEqual(1, set.number())

        except Exception as e:
            self.fail(e)

    def test_ly_data_node_validate(self):
        yang_folder = config.TESTS_DIR + "/api/files"
        config_file = config.TESTS_DIR + "/api/files/a.xml"
        try:
            # Setup
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)
            ctx.parse_module_mem(lys_module_a, ly.LYS_IN_YIN)
            root = ctx.parse_data_path(config_file, ly.LYD_XML, ly.LYD_OPT_CONFIG | ly.LYD_OPT_STRICT)
            self.assertIsNotNone(root)

            # Tests
            rc = root.validate(ly.LYD_OPT_CONFIG, ctx)
            self.assertEqual(0, rc)
            new = ly.Data_Node(root, root.schema().module(), "number32", "1")
            self.assertIsNotNone(new)
            rc = root.insert(new)
            self.assertEqual(0, rc)
            rc = root.validate(ly.LYD_OPT_CONFIG, ctx)
            self.assertEqual(0, rc)

        except Exception as e:
            self.fail(e)

    def test_ly_data_node_unlink(self):
        yang_folder = config.TESTS_DIR + "/api/files"
        config_file = config.TESTS_DIR + "/api/files/a.xml"
        try:
            # Setup
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)
            ctx.parse_module_mem(lys_module_a, ly.LYS_IN_YIN)
            root = ctx.parse_data_path(config_file, ly.LYD_XML, ly.LYD_OPT_CONFIG | ly.LYD_OPT_STRICT)
            self.assertIsNotNone(root)

            # Tests
            node = root.child()
            new = ly.Data_Node(root, node.schema().module(), "number32", "1")
            self.assertIsNotNone(new)
            rc = root.insert(new)
            self.assertEqual(0, rc)

            schema = node.prev().schema()
            if (ly.LYS_LEAF == schema.nodetype() or ly.LYS_LEAFLIST == schema.nodetype()):
                casted = node.prev().subtype()
                self.assertEqual("1", casted.value_str())
            else:
                self.fail()

            rc = node.prev().unlink()
            self.assertEqual(0, rc)
            schema = node.prev().schema()
            if (ly.LYS_LEAF == schema.nodetype() or ly.LYS_LEAFLIST == schema.nodetype()):
                self.fail()
            else:
                return

        except Exception as e:
            self.fail(e)

    def test_ly_data_node_print_mem_xml(self):
        yang_folder = config.TESTS_DIR + "/api/files"
        config_file = config.TESTS_DIR + "/api/files/a.xml"
        try:
            # Setup
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)
            ctx.parse_module_mem(lys_module_a, ly.LYS_IN_YIN)
            root = ctx.parse_data_path(config_file, ly.LYD_XML, ly.LYD_OPT_CONFIG | ly.LYD_OPT_STRICT)
            self.assertIsNotNone(root)

            # Tests
            result = root.print_mem(ly.LYD_XML, 0)
            self.assertEqual(result_xml, result)

        except Exception as e:
            self.fail(e)

    def test_ly_data_node_print_mem_xml_format(self):
        yang_folder = config.TESTS_DIR + "/api/files"
        config_file = config.TESTS_DIR + "/api/files/a.xml"
        try:
            # Setup
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)
            ctx.parse_module_mem(lys_module_a, ly.LYS_IN_YIN)
            root = ctx.parse_data_path(config_file, ly.LYD_XML, ly.LYD_OPT_CONFIG | ly.LYD_OPT_STRICT)
            self.assertIsNotNone(root)

            # Tests
            result = root.print_mem(ly.LYD_XML, ly.LYP_FORMAT)
            self.assertEqual(result_xml_format, result)


        except Exception as e:
            self.fail(e)

    def test_ly_data_node_print_mem_json(self):
        yang_folder = config.TESTS_DIR + "/api/files"
        config_file = config.TESTS_DIR + "/api/files/a.xml"
        try:
            # Setup
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)
            ctx.parse_module_mem(lys_module_a, ly.LYS_IN_YIN)
            root = ctx.parse_data_path(config_file, ly.LYD_XML, ly.LYD_OPT_CONFIG | ly.LYD_OPT_STRICT)
            self.assertIsNotNone(root)

            # Tests
            result = root.print_mem(ly.LYD_JSON, ly.LYP_FORMAT)
            self.assertEqual(result_json, result)

        except Exception as e:
            self.fail(e)

    def test_ly_data_node_path(self):
        yang_folder = config.TESTS_DIR + "/api/files"
        config_file = config.TESTS_DIR + "/api/files/a.xml"
        try:
            # Setup
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)
            ctx.parse_module_mem(lys_module_a, ly.LYS_IN_YIN)
            root = ctx.parse_data_path(config_file, ly.LYD_XML, ly.LYD_OPT_CONFIG | ly.LYD_OPT_STRICT)
            self.assertIsNotNone(root)

            # Tests
            str = root.path()
            self.assertIsNotNone(str)
            self.assertEqual("/a:x", str)
            str = root.child().path()
            self.assertIsNotNone(str)
            self.assertEqual("/a:x/bubba", str)

        except Exception as e:
            self.fail(e)

    def test_ly_data_node_leaf(self):
        yang_folder = config.TESTS_DIR + "/api/files"
        config_file = config.TESTS_DIR + "/api/files/a.xml"

        try:
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)
            ctx.parse_module_mem(lys_module_a, ly.LYS_IN_YIN)
            root = ctx.parse_data_path(config_file, ly.LYD_XML, ly.LYD_OPT_CONFIG | ly.LYD_OPT_STRICT)
            self.assertIsNotNone(root)

            new_node = ly.Data_Node(root, root.schema().module(), "number32", "100")
            self.assertIsNotNone(new_node)

        except Exception as e:
            self.fail(e)

    def test_ly_data_node_anydata(self):
        yang_folder = config.TESTS_DIR + "/api/files"
        config_file = config.TESTS_DIR + "/api/files/a.xml"

        try:
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)
            ctx.parse_module_mem(lys_module_a, ly.LYS_IN_YIN)
            root = ctx.parse_data_path(config_file, ly.LYD_XML, ly.LYD_OPT_CONFIG | ly.LYD_OPT_STRICT)
            self.assertIsNotNone(root)
            mod = ctx.get_module("a", None, 1)

            new_node = ly.Data_Node(root, mod, "any-data", "100", ly.LYD_ANYDATA_CONSTSTRING)
            self.assertIsNotNone(new_node)

        except Exception as e:
            self.fail(e)

    def test_ly_data_node_dup(self):
        yang_folder = config.TESTS_DIR + "/api/files";
        config_file = config.TESTS_DIR + "/api/files/a.xml";

        try:
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)
            ctx.parse_module_mem(lys_module_a, ly.LYS_IN_YIN)
            root = ctx.parse_data_path(config_file, ly.LYD_XML, ly.LYD_OPT_CONFIG | ly.LYD_OPT_STRICT)
            self.assertIsNotNone(root)

            new_node = ly.Data_Node(root, root.child().schema().module(), "bar-y")
            self.assertIsNotNone(new_node)
            dup_node = new_node.dup(0);
            self.assertIsNotNone(dup_node)

        except Exception as e:
            self.fail(e)


    def test_ly_data_node_dup_to_ctx(self):
        sch = "module x {\
              namespace urn:x;\
              prefix x;\
              leaf x { type string; }}"
        data = "<x xmlns=\"urn:x\">hello</x>"

        try:
            ctx1 = ly.Context(None)
            self.assertIsNotNone(ctx1);
            ctx1.parse_module_mem(sch, ly.LYS_IN_YANG)
            data1 = ctx1.parse_data_mem(data, ly.LYD_XML, ly.LYD_OPT_CONFIG | ly.LYD_OPT_STRICT)
            self.assertIsNotNone(data1)

            ctx2 = ly.Context(None)
            self.assertIsNotNone(ctx2)
            # we expect NULL due to missing schema in the second ctx
            dup_node = data1.dup_to_ctx(1, ctx2)
            self.assertIsNone(dup_node)

            ctx2.parse_module_mem(sch, ly.LYS_IN_YANG)
            # now we expect success due to schema being added to the second ctx
            dup_node = data1.dup_to_ctx(1, ctx2)
            self.assertIsNotNone(dup_node)

        except Exception as e:
            self.fail(e)

    def test_ly_data_node_validate_node(self):
        yang_folder = config.TESTS_DIR + "/api/files";
        config_file = config.TESTS_DIR + "/api/files/a.xml";

        try:
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)
            ctx.parse_module_mem(lys_module_a, ly.LYS_IN_YIN)
            root = ctx.parse_data_path(config_file, ly.LYD_XML, ly.LYD_OPT_CONFIG | ly.LYD_OPT_STRICT)
            self.assertIsNotNone(root)

            rc = root.validate(ly.LYD_OPT_CONFIG, ctx)
            self.assertEqual(0, rc)
            new_node = ly.Data_Node(root, root.schema().module(), "number32", "1")
            self.assertIsNotNone(new_node)
            rc = root.validate(ly.LYD_OPT_CONFIG, new_node)
            self.assertEqual(0, rc)

        except Exception as e:
            self.fail(e)

    def test_ly_data_node_validate_value(self):
        yang_folder = config.TESTS_DIR + "/api/files";
        config_file = config.TESTS_DIR + "/api/files/a.xml";

        try:
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)
            ctx.parse_module_mem(lys_module_a, ly.LYS_IN_YIN)
            root = ctx.parse_data_path(config_file, ly.LYD_XML, ly.LYD_OPT_CONFIG | ly.LYD_OPT_STRICT)
            self.assertIsNotNone(root)

            rc = root.validate(ly.LYD_OPT_CONFIG, ctx)
            self.assertEqual(0, rc)
            new_node = ly.Data_Node(root, root.schema().module(), "number32", "1")
            self.assertIsNotNone(new_node)
            self.assertEqual(0, new_node.validate_value("1"))
            self.assertEqual(0, new_node.validate_value("100"))
            self.assertEqual(0, new_node.validate_value("110000000"))

        except Exception as e:
            self.fail(e)

if __name__ == '__main__':
    unittest.main()



__author__ = "Mislav Novakovic <mislav.novakovic@sartura.hr>"
__copyright__ = "Copyright 2017, Deutsche Telekom AG"
__license__ = "BSD 3-Clause"

import yang as ly

ctx = None
try:
    ctx = ly.Context("/etc/sysrepo/yang")
except Exception as e:
    print(e)
    errors = ly.get_ly_errors(ctx)
    for err in errors:
        print("err: %d" % err.err())
        print("vecode: %d" % err.vecode())
        print("errmsg: " + err.errmsg())
        print("errpath:" + err.errpath())
        print("errapptag:" + err.errapptag())

module = ctx.get_module("turing-machine")
if module is not None:
    print(module.name())
else:
    module = ctx.load_module("turing-machine")

node = None
try:
    if node is None : node = ctx.parse_data_path("/etc/sysrepo/data/turing-machine.startup", ly.LYD_LYB, ly.LYD_OPT_CONFIG)
except Exception as e:
    print(e)
try:
    if node is None : node = ctx.parse_data_path("/etc/sysrepo/data/turing-machine.startup", ly.LYD_XML, ly.LYD_OPT_CONFIG)
except Exception as e:
    print(e)
try:
    if node is None : node = ctx.parse_data_path("/etc/sysrepo/data/turing-machine.startup", ly.LYD_JSON, ly.LYD_OPT_CONFIG)
except Exception as e:
    print(e)

if node is None:
    sys.exit()

if node is None:
    print("parse_data_path did not return any nodes")
else:
    print("tree_dfs\n")
    data_list = node.tree_dfs()
    for elem in data_list:
        schema = elem.schema()
        print("name: " + schema.name() + " type: " + str(schema.nodetype()))
        if (ly.LYS_LEAF == schema.nodetype() or ly.LYS_LEAFLIST == schema.nodetype()):
            casted = elem.subtype()
            if casted is None:
                continue
            print("node " + casted.schema().name() + " has value " + casted.value_str())

    print("\nChild of " + node.schema().name() + " is " + node.child().schema().name() + " \n ")

    print("tree_for\n")
    data_list = node.child().child().tree_for()
    for elem in data_list:
        print("child of " + node.child().schema().name() + " is: " + elem.schema().name() + " type: " + str(elem.schema().nodetype()))

    print("\nschema tree_dfs\n")
    schema_list = node.schema().tree_dfs()
    for elem in schema_list:
        print("schema name: " + elem.name() + " type: " + str(elem.nodetype()))



#!/usr/bin/env python
from __future__ import print_function

__author__ = "Matija Amidzic <matija.amidzic@sartura.hr>"
__copyright__ = "Copyright 2018, Deutsche Telekom AG"
__license__ = "BSD 3-Clause"

# This source code is licensed under BSD 3-Clause License (the "License").
# You may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://opensource.org/licenses/BSD-3-Clause

import yang as ly
import unittest
import sys

import config

class UnexpectedError(Exception):
    """Exception raised for errors that are not expected.

    Attributes:
        message -- explanation of the error
    """

    def __init__(self, message):
        self.message = message

class TestUM(unittest.TestCase):
    def test_ly_ctx_new(self):
        yang_folder1 = config.TESTS_DIR + "/data/files"
        yang_folder2 = config.TESTS_DIR + "/data:" + config.TESTS_DIR + "/data/files"

        try:
            # Tests
            ctx = ly.Context(yang_folder1)
            self.assertIsNotNone(ctx)
            list = ctx.get_searchdirs()
            self.assertIsNotNone(list)
            self.assertEqual(1, len(list))

            ctx = ly.Context(yang_folder2)
            self.assertIsNotNone(ctx)
            list = ctx.get_searchdirs()
            self.assertIsNotNone(list)
            self.assertEqual(2, len(list))

        except Exception as e:
            self.fail(e)

    def test_ly_ctx_new_invalid(self):
        yang_folder = "INVALID_PATH"

        try:
            ctx = ly.Context(yang_folder)
            raise UnexpectedError("exception not thrown")

        except UnexpectedError as e:
            self.fail(e)

        except RuntimeError as e:
            return

        except Exception as e:
            self.fail(e)

    def test_ly_ctx_get_searchdirs(self):
        yang_folder = config.TESTS_DIR + "/data/files"

        try:
            # Setup
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)

            # Tests
            list = ctx.get_searchdirs()
            self.assertEqual(1, len(list))
            self.assertEqual(yang_folder, list[0])

        except Exception as e:
            self.fail(e)

    def test_ly_ctx_set_searchdir(self):
        yang_folder = config.TESTS_DIR + "/data/files"
        new_yang_folder = config.TESTS_DIR + "/schema/yin"

        try:
            # Setup
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)

            # Tests
            list = ctx.get_searchdirs()
            self.assertEqual(1, len(list))
            self.assertEqual(yang_folder, list[0])

            ctx.set_searchdir(new_yang_folder)
            list = ctx.get_searchdirs()
            self.assertEqual(2, len(list))
            self.assertEqual(new_yang_folder, list[1])

            ctx.unset_searchdirs(0)
            list = ctx.get_searchdirs()
            self.assertEqual(1, len(list))
            self.assertEqual(new_yang_folder, list[0])

        except Exception as e:
            self.fail(e)

    def test_ly_ctx_set_searchdir_invalid(self):
        yang_folder = config.TESTS_DIR + "/data/files"
        new_yang_folder = "INVALID_PATH"

        try:
            # Setup
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)

            # Tests
            ctx.set_searchdir(new_yang_folder)
            raise UnexpectedError("exception not thrown")

        except UnexpectedError as e:
            self.fail(e)

        except RuntimeError as e:
            return

        except Exception as e:
            self.fail(e)

    def test_ly_ctx_info(self):
        yang_folder = config.TESTS_DIR + "/api/files"

        try:
            # Setup
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)

            # Tests
            info = ctx.info()
            self.assertIsNotNone(info)
            self.assertEqual(ly.LYD_VAL_OK, info.validity())

        except Exception as e:
            self.fail(e)

    def test_ly_ctx_load_module_invalid(self):
        yang_folder = config.TESTS_DIR + "/api/files"

        try:
            # Setup
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)

            # Tests
            module = ctx.load_module("invalid", None)
            raise UnexpectedError("exception not thrown")

        except UnexpectedError as e:
            self.fail(e)

        except RuntimeError as e:
            return

        except Exception as e:
            self.fail(e)

    def test_ly_ctx_load_get_module(self):
        yang_folder = config.TESTS_DIR + "/api/files"
        name1 = "a"
        name2 = "b"
        revision = "2016-03-01"

        try:
            # Setup
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)

            # Tests
            module = ctx.get_module("invalid")
            self.assertIsNone(module)

            # module needs to be loaded first
            module = ctx.get_module(name1)
            self.assertIsNone(module)

            module = ctx.load_module(name1)
            self.assertIsNotNone(module)
            self.assertEqual(name1, module.name(), "Module names don't match")

            module = ctx.load_module(name2, revision)
            self.assertIsNotNone(module)
            self.assertEqual(name2, module.name(), "Module names don't match")
            self.assertEqual(revision, module.rev().date(), "Module revisions don't match")

            module = ctx.get_module(name2, "INVALID_REVISION")
            self.assertIsNone(module)

            module = ctx.get_module(name1)
            self.assertIsNotNone(module)
            self.assertEqual(name1, module.name(), "Module names don't match")

            module = ctx.get_module(name2, revision)
            self.assertIsNotNone(module)
            self.assertEqual(name2, module.name(), "Module names don't match")
            self.assertEqual(revision, module.rev().date(), "Module revisions don't match")

        except Exception as e:
            self.fail(e)

    def test_ly_ctx_get_module_older(self):
        yang_folder = config.TESTS_DIR + "/api/files"
        name = "b"
        revision = "2016-03-01"
        revision_older = "2015-01-01"

        try:
            # Setup
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)

            # Tests
            module = ctx.load_module("c")
            self.assertIsNotNone(module)
            self.assertEqual("c", module.name(), "Module names don't match")

            module = ctx.load_module(name, revision)
            self.assertIsNotNone(module)
            self.assertEqual(name, module.name(), "Module names don't match")
            self.assertEqual(revision, module.rev().date(), "Module revisions don't match")

            module_older = ctx.get_module_older(module)
            self.assertIsNotNone(module_older)
            self.assertEqual(name, module_older.name(), "Module names don't match")
            self.assertEqual(revision_older, module_older.rev().date(), "Module revisions don't match")

        except Exception as e:
            self.fail(e)

    def test_ly_ctx_get_module_by_ns(self):
        yang_folder = config.TESTS_DIR + "/api/files"
        module_name = "a"
        ns = "urn:a"

        try:
            # Setup
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)
            module = ctx.load_module(module_name)
            self.assertIsNotNone(module)
            self.assertEqual(module_name, module.name(), "Module names don't match")

            # Tests
            module = ctx.get_module_by_ns(ns)
            self.assertIsNotNone(module)
            self.assertEqual(module_name, module.name(), "Module names don't match")

        except Exception as e:
            self.fail(e)

    def test_ly_ctx_clean(self):
        yang_folder = config.TESTS_DIR + "/api/files"
        module_name = "a"

        try:
            # Setup
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)
            module = ctx.load_module(module_name)
            self.assertIsNotNone(module)
            self.assertEqual(module_name, module.name(), "Module names don't match")

            # Tests
            # confirm module is loaded
            module = ctx.get_module(module_name)
            self.assertIsNotNone(module)
            self.assertEqual(module_name, module.name(), "Module names don't match")

            ctx.clean()

            # confirm ctx is cleaned
            module = ctx.get_module(module_name)
            self.assertIsNone(module)

        except Exception as e:
            self.fail(e)

    def test_ly_ctx_parse_module_path(self):
        yang_folder = config.TESTS_DIR + "/api/files"
        yin_file = config.TESTS_DIR + "/api/files/a.yin"
        yang_file = config.TESTS_DIR + "/api/files/b.yang"
        module_name1 = "a"
        module_name2 = "b"

        try:
            # Setup
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)

            # Tests
            module = ctx.parse_module_path(yin_file, ly.LYS_IN_YIN)
            self.assertIsNotNone(module)
            self.assertEqual(module_name1, module.name(), "Module names don't match")

            module = ctx.parse_module_path(yang_file, ly.LYS_IN_YANG)
            self.assertIsNotNone(module)
            self.assertEqual(module_name2, module.name(), "Module names don't match")

        except Exception as e:
            self.fail(e)

    def test_ly_ctx_parse_module_path_invalid(self):
        yang_folder = config.TESTS_DIR + "/api/files"

        try:
            # Setup
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)

            # Tests
            module = ctx.parse_module_path("INVALID_YANG_FILE", ly.LYS_IN_YANG)
            raise UnexpectedError("exception not thrown")

        except UnexpectedError as e:
            self.fail(e)

        except RuntimeError as e:
            return

        except Exception as e:
            self.fail(e)

    def test_ly_ctx_get_submodule(self):
        yang_folder = config.TESTS_DIR + "/api/files"
        yin_file = config.TESTS_DIR + "/api/files/a.yin"
        module_name = "a"
        sub_name = "asub"

        try:
            # Setup
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)
            ctx.parse_module_path(yin_file, ly.LYS_IN_YIN)

            # Tests
            submodule = ctx.get_submodule(module_name, None, sub_name, None)
            self.assertIsNotNone(submodule)
            self.assertEqual(sub_name, submodule.name(), "Module names don't match")

        except Exception as e:
            self.fail(e)

    def test_ly_ctx_get_submodule2(self):
        yang_folder = config.TESTS_DIR + "/api/files"
        yin_file = config.TESTS_DIR + "/api/files/a.yin"
        config_file = config.TESTS_DIR + "/api/files/a.xml"
        sub_name = "asub"

        try:
            # Setup
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)
            ctx.parse_module_path(yin_file, ly.LYS_IN_YIN)
            root = ctx.parse_data_path(config_file, ly.LYD_XML, ly.LYD_OPT_CONFIG | ly.LYD_OPT_STRICT)
            self.assertIsNotNone(root)
            self.assertIsNotNone(root.schema().module())

            # Tests
            submodule = ctx.get_submodule2(root.schema().module(), sub_name)
            self.assertIsNotNone(submodule)
            self.assertEqual(sub_name, submodule.name(), "Module names don't match")

        except Exception as e:
            self.fail(e)

    def test_ly_ctx_find_path(self):
        yang_folder = config.TESTS_DIR + "/api/files"
        yin_file = config.TESTS_DIR + "/api/files/a.yin"
        yang_file = config.TESTS_DIR + "/api/files/b.yang"
        schema_path1 = "/b:x/b:bubba"
        schema_path2 = "/a:x/a:bubba"

        try:
            # Setup
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)

            # Tests
            ctx.parse_module_path(yang_file, ly.LYS_IN_YANG)
            set = ctx.find_path(schema_path1)
            self.assertIsNotNone(set)

            ctx.parse_module_path(yin_file, ly.LYS_IN_YIN)
            set = ctx.find_path(schema_path2)
            self.assertIsNotNone(set)
            ly.Set()

        except Exception as e:
            self.fail(e)

    def test_ly_set(self):
        yang_folder = config.TESTS_DIR + "/api/files"
        yin_file = config.TESTS_DIR + "/api/files/a.yin"
        config_file = config.TESTS_DIR + "/api/files/a.xml"

        try:
            # Setup
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)
            ctx.parse_module_path(yin_file, ly.LYS_IN_YIN)
            root = ctx.parse_data_path(config_file, ly.LYD_XML, ly.LYD_OPT_CONFIG | ly.LYD_OPT_STRICT)
            self.assertIsNotNone(root)

            # Tests
            set = ly.Set()
            self.assertIsNotNone(set)
            self.assertEqual(0, set.number())

            set.add(root.child().schema())
            self.assertEqual(1, set.number())

            set.add(root.schema())
            self.assertEqual(2, set.number())

            set.rm(root.schema())
            self.assertEqual(1, set.number())

            set.add(root.schema())
            self.assertEqual(2, set.number())

            set.rm_index(1)
            self.assertEqual(1, set.number())

            set.clean()
            self.assertEqual(0, set.number())

        except Exception as e:
            self.fail(e)

    def test_ly_ctx_new_ylpath(self):
        yang_folder = config.TESTS_DIR + "/api/files"
        path = config.TESTS_DIR + "/api/files/ylpath.xml"

        try:
            ctx = ly.Context(yang_folder, path, ly.LYD_XML, 0)
            self.assertIsNotNone(ctx)
            list = ctx.get_searchdirs()
            self.assertIsNotNone(list)
            self.assertEqual(1, len(list))

        except Exception as e:
            self.fail(e)

    def test_ly_ctx_new_ylmem(self):
        yang_folder = config.TESTS_DIR + "/api/files"

        try:
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)
            info = ctx.info()
            self.assertIsNotNone(info)
            self.assertEqual(ly.LYD_VAL_OK, info.validity())

            mem = info.print_mem(ly.LYD_XML, 0)
            self.assertIsNotNone(mem)

            new_ctx = ly.Context(yang_folder, ly.LYD_XML, mem, 0)
            self.assertIsNotNone(new_ctx)
            list = ctx.get_searchdirs()
            self.assertIsNotNone(list)
            self.assertEqual(1, len(list))

        except Exception as e:
            self.fail(e)

    def test_ly_ctx_get_module_iter(self):
        yang_folder = config.TESTS_DIR + "/api/files"
        module_name = "a"

        try:
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)

            module = ctx.load_module(module_name)
            self.assertIsNotNone(module)
            self.assertEqual(module_name, module.name())
            itr = ctx.get_module_iter()
            self.assertIsNotNone(itr)
            self.assertEqual(7, len(itr))

        except Exception as e:
            self.fail(e)


    def test_ly_ctx_get_disabled_module_iter(self):
        yang_folder = config.TESTS_DIR +  "/api/files"
        module_name = "x"

        try:
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)

            module = ctx.load_module(module_name)
            self.assertIsNotNone(module)
            self.assertEqual(module_name, module.name())
            # FIXME no way to disable module from here

            itr = ctx.get_disabled_module_iter()
            self.assertIsNotNone(itr)
            self.assertEqual(0, len(itr))

        except Exception as e:
            self.fail(e)

    def test_ly_ctx_data_instantiables(self):
        yang_folder = config.TESTS_DIR + "/api/files"
        module_name = "b"

        try:
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)

            module = ctx.load_module(module_name)
            self.assertIsNotNone(module)
            self.assertEqual(module_name, module.name())

            instantiables = ctx.data_instantiables(0)
            self.assertIsNotNone(instantiables)
            self.assertEqual(5, len(instantiables))

        except Exception as e:
            self.fail(e)

    def test_ly_ctx_get_node(self):
        yang_folder = config.TESTS_DIR + "/api/files"
        module_name = "b"

        try:
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)

            module = ctx.load_module(module_name)
            self.assertIsNotNone(module)
            self.assertEqual(module_name, module.name())

            instantiables = ctx.data_instantiables(0)
            self.assertIsNotNone(instantiables)
            self.assertEqual(5, len(instantiables))

            schema = instantiables[0];
            self.assertIsNotNone(schema)
            node = ctx.get_node(schema, "/b:x/b:bubba", 0)
            self.assertIsNotNone(node)

        except Exception as e:
            self.fail(e)

if __name__ == '__main__':
    unittest.main()



__author__ = "Mislav Novakovic <mislav.novakovic@sartura.hr>"
__copyright__ = "Copyright 2017, Deutsche Telekom AG"
__license__ = "BSD 3-Clause"

import yang as ly

mod_a = "module a {namespace urn:a; prefix a; import b { prefix b; } leaf a { type b:mytype; }}"
mod_b = "module b {namespace urn:b; prefix b; typedef mytype { type string; }}"

def module_imp_clb(mod_name, mod_rev, submod_name, sub_rev, user_data):
    print("module_imp_cb called")
    print("mod_name: ", mod_name, " mod_rev: ", mod_rev, " submod_name: ", submod_name, " sub_rev: ", sub_rev)
    return (ly.LYS_IN_YANG, mod_b)


ctx = None
try:
    ctx = ly.Context()
except Exception as e:
    print(e)

ctx.set_module_imp_clb(module_imp_clb, None)
module = ctx.parse_module_mem(mod_a, ly.LYS_IN_YANG)
print(module.print_mem(ly.LYS_OUT_TREE, 0))



#!/usr/bin/env python
from __future__ import print_function

__author__ = "Matija Amidzic <matija.amidzic@sartura.hr>"
__copyright__ = "Copyright 2018, Deutsche Telekom AG"
__license__ = "BSD 3-Clause"

# This source code is licensed under BSD 3-Clause License (the "License").
# You may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://opensource.org/licenses/BSD-3-Clause

import yang as ly
import unittest
import sys

import config

lys_module_a = \
"<?xml version=\"1.0\" encoding=\"UTF-8\"?>           \
<module name=\"a\"                                    \
        xmlns=\"urn:ietf:params:xml:ns:yang:yin:1\"   \
        xmlns:a=\"urn:a\">                            \
  <namespace uri=\"urn:a\"/>                          \
  <prefix value=\"a_mod\"/>                           \
  <include module=\"asub\"/>                          \
  <include module=\"atop\"/>                          \
  <revision date=\"2015-01-01\">                      \
    <description>                                     \
      <text>version 1</text>                          \
    </description>                                    \
    <reference>                                       \
      <text>RFC XXXX</text>                           \
    </reference>                                      \
  </revision>                                         \
  <feature name=\"foo\"/>                             \
  <grouping name=\"gg\">                              \
    <leaf name=\"bar-gggg\">                          \
      <type name=\"string\"/>                         \
    </leaf>                                           \
  </grouping>                                         \
  <container name=\"x\">                              \
    <leaf name=\"bar-leaf\">                          \
      <if-feature name=\"bar\"/>                      \
      <type name=\"string\"/>                         \
    </leaf>                                           \
    <uses name=\"gg\">                                \
      <if-feature name=\"bar\"/>                      \
    </uses>                                           \
    <leaf name=\"baz\">                               \
      <if-feature name=\"foo\"/>                      \
      <type name=\"string\"/>                         \
    </leaf>                                           \
    <leaf name=\"bubba\">                             \
      <type name=\"string\"/>                         \
    </leaf>                                           \
  </container>                                        \
  <augment target-node=\"/x\">                        \
    <if-feature name=\"bar\"/>                        \
    <container name=\"bar-y\">                        \
      <leaf name=\"ll\">                              \
        <type name=\"string\"/>                       \
      </leaf>                                         \
    </container>                                      \
  </augment>                                          \
  <rpc name=\"bar-rpc\">                              \
    <if-feature name=\"bar\"/>                        \
  </rpc>                                              \
  <rpc name=\"foo-rpc\">                              \
    <if-feature name=\"foo\"/>                        \
  </rpc>                                              \
</module>                                             \
"

lys_module_b = \
"module b {\
    namespace \"urn:b\";\
    prefix b_mod;\
    include bsub;\
    include btop;\
    feature foo;\
    grouping gg {\
        leaf bar-gggg {\
            type string;\
        }\
    }\
    container x {\
        leaf bar-leaf {\
            if-feature \"bar\";\
            type string;\
        }\
        uses gg {\
            if-feature \"bar\";\
        }\
        leaf baz {\
            if-feature \"foo\";\
            type string;\
        }\
        leaf bubba {\
            type string;\
        }\
    }\
    augment \"/x\" {\
        if-feature \"bar\";\
        container bar-y;\
    }\
    rpc bar-rpc {\
        if-feature \"bar\";\
    }\
    rpc foo-rpc {\
        if-feature \"foo\";\
    }\
}"

lys_module_a_with_typo = \
"<?xml version=\"1.0\" encoding=\"UTF-8\"?>           \
<module_typo name=\"a\"                                    \
        xmlns=\"urn:ietf:params:xml:ns:yang:yin:1\"   \
        xmlns:a=\"urn:a\">                            \
  <namespace uri=\"urn:a\"/>                          \
  <prefix value=\"a_mod\"/>                           \
  <include module=\"asub\"/>                          \
  <include module=\"atop\"/>                          \
  <feature name=\"foo\"/>                             \
  <grouping name=\"gg\">                              \
    <leaf name=\"bar-gggg\">                          \
      <type name=\"string\"/>                         \
    </leaf>                                           \
  </grouping>                                         \
  <container name=\"x\">                              \
    <leaf name=\"bar-leaf\">                          \
      <if-feature name=\"bar\"/>                      \
      <type name=\"string\"/>                         \
    </leaf>                                           \
    <uses name=\"gg\">                                \
      <if-feature name=\"bar\"/>                      \
    </uses>                                           \
    <leaf name=\"baz\">                               \
      <if-feature name=\"foo\"/>                      \
      <type name=\"string\"/>                         \
    </leaf>                                           \
    <leaf name=\"bubba\">                             \
      <type name=\"string\"/>                         \
    </leaf>                                           \
  </container>                                        \
  <augment target-node=\"/x\">                        \
    <if-feature name=\"bar\"/>                        \
    <container name=\"bar-y\">                        \
      <leaf name=\"ll\">                              \
        <type name=\"string\"/>                       \
      </leaf>                                         \
    </container>                                      \
  </augment>                                          \
  <rpc name=\"bar-rpc\">                              \
    <if-feature name=\"bar\"/>                        \
  </rpc>                                              \
  <rpc name=\"foo-rpc\">                              \
    <if-feature name=\"foo\"/>                        \
  </rpc>                                              \
</module>                                             \
"

result_tree = "\
module: a\n\
  +--rw top\n\
  |  +--rw bar-sub2\n\
  +--rw x\n\
     +--rw bubba?      string\n"

result_yang = "\
module a {\n\
  namespace \"urn:a\";\n\
  prefix a_mod;\n\
\n\
  include \"asub\";\n\
\n\
  include \"atop\";\n\
\n\
  revision 2015-01-01 {\n\
    description\n\
      \"version 1\";\n\
    reference\n\
      \"RFC XXXX\";\n\
  }\n\
\n\
  feature foo;\n\
\n\
  grouping gg {\n\
    leaf bar-gggg {\n\
      type string;\n\
    }\n\
  }\n\
\n\
  container x {\n\
    leaf bar-leaf {\n\
      if-feature \"bar\";\n\
      type string;\n\
    }\n\n\
    uses gg {\n\
      if-feature \"bar\";\n\
    }\n\n\
    leaf baz {\n\
      if-feature \"foo\";\n\
      type string;\n\
    }\n\n\
    leaf bubba {\n\
      type string;\n\
    }\n\
  }\n\
\n\
  augment \"/x\" {\n\
    if-feature \"bar\";\n\
    container bar-y {\n\
      leaf ll {\n\
        type string;\n\
      }\n\
    }\n\
  }\n\
\n\
  rpc bar-rpc {\n\
    if-feature \"bar\";\n\
  }\n\
\n\
  rpc foo-rpc {\n\
    if-feature \"foo\";\n\
  }\n\
}\n"

result_yin = "\
<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n\
<module name=\"a\"\n\
        xmlns=\"urn:ietf:params:xml:ns:yang:yin:1\"\n\
        xmlns:a_mod=\"urn:a\">\n\
  <namespace uri=\"urn:a\"/>\n\
  <prefix value=\"a_mod\"/>\n\
  <include module=\"asub\"/>\n\
  <include module=\"atop\"/>\n\
  <revision date=\"2015-01-01\">\n\
    <description>\n\
      <text>version 1</text>\n\
    </description>\n\
    <reference>\n\
      <text>RFC XXXX</text>\n\
    </reference>\n\
  </revision>\n\
  <feature name=\"foo\"/>\n\
  <grouping name=\"gg\">\n\
    <leaf name=\"bar-gggg\">\n\
      <type name=\"string\"/>\n\
    </leaf>\n\
  </grouping>\n\
  <container name=\"x\">\n\
    <leaf name=\"bar-leaf\">\n\
      <if-feature name=\"bar\"/>\n\
      <type name=\"string\"/>\n\
    </leaf>\n\
    <uses name=\"gg\">\n\
      <if-feature name=\"bar\"/>\n\
    </uses>\n\
    <leaf name=\"baz\">\n\
      <if-feature name=\"foo\"/>\n\
      <type name=\"string\"/>\n\
    </leaf>\n\
    <leaf name=\"bubba\">\n\
      <type name=\"string\"/>\n\
    </leaf>\n\
  </container>\n\
  <augment target-node=\"/x\">\n\
    <if-feature name=\"bar\"/>\n\
    <container name=\"bar-y\">\n\
      <leaf name=\"ll\">\n\
        <type name=\"string\"/>\n\
      </leaf>\n\
    </container>\n\
  </augment>\n\
  <rpc name=\"bar-rpc\">\n\
    <if-feature name=\"bar\"/>\n\
  </rpc>\n\
  <rpc name=\"foo-rpc\">\n\
    <if-feature name=\"foo\"/>\n\
  </rpc>\n\
</module>\n"

result_info ="\
Feature:   foo\n\
Module:    a\n\
Desc:      \n\
Reference: \n\
Status:    current\n\
Enabled:   no\n\
If-feats:  \n"

class UnexpectedError(Exception):
    """Exception raised for errors that are not expected.

    Attributes:
        message -- explanation of the error
    """

    def __init__(self, message):
        self.message = message

class TestUM(unittest.TestCase):
    def test_ly_ctx_parse_module_mem(self):
        yang_folder = config.TESTS_DIR + "/api/files"

        try:
            # Setup
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)

            # Tests
            module = ctx.parse_module_mem(lys_module_a, ly.LYS_IN_YIN)
            self.assertIsNotNone(module)
            self.assertEqual("a", module.name())

            module = ctx.parse_module_mem(lys_module_b, ly.LYS_IN_YANG)
            self.assertIsNotNone(module)
            self.assertEqual("b", module.name())

        except Exception as e:
            self.fail(e)

    def test_ly_ctx_parse_module_mem_invalid(self):
        yang_folder = config.TESTS_DIR + "/api/files"

        try:
            # Setup
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)

            # Tests
            ctx.parse_module_mem(lys_module_a_with_typo, ly.LYS_IN_YIN)
            raise UnexpectedError("Exception not thrown")

        except UnexpectedError as e:
            self.fail(e)

        except RuntimeError as e:
            return

        except Exception as e:
            self.fail(e)

    def test_ly_ctx_parse_module_fd(self):
        yang_folder = config.TESTS_DIR + "/api/files"
        yin_file = config.TESTS_DIR + "/api/files/a.yin"
        yang_file = config.TESTS_DIR + "/api/files/b.yang"

        try:
            # Setup
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)

            # Tests
            f = open(yin_file, 'r')
            fd = f.fileno()
            module = ctx.parse_module_fd(fd, ly.LYS_IN_YIN)
            self.assertIsNotNone(module)
            self.assertEqual("a", module.name())

            f.close()
            f = open(yang_file, 'r')
            fd = f.fileno()
            module = ctx.parse_module_fd(fd, ly.LYS_IN_YANG)
            self.assertIsNotNone(module)
            self.assertEqual("b", module.name())

        except Exception as e:
            self.fail(e)

        finally:
            f.close()

    def test_ly_ctx_parse_module_fd_invalid(self):
        yang_folder = config.TESTS_DIR + "/api/files"
        yin_file = config.TESTS_DIR + "/api/files/a.yin"

        try:
            # Setup
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)

            # Tests
            f = open(yin_file, 'r')
            fd = f.fileno()
            # parsing with wrong format should raise runtime exception
            module = ctx.parse_module_fd(fd, ly.LYS_IN_YANG)
            raise UnexpectedError("Exception not thrown")

        except UnexpectedError as e:
            self.fail(e)

        except RuntimeError as e:
            return

        except Exception as e:
            self.fail(e)

        finally:
            f.close()

    def test_ly_ctx_parse_module_path(self):
        yang_folder = config.TESTS_DIR + "/api/files"
        yin_file = config.TESTS_DIR + "/api/files/a.yin"
        yang_file = config.TESTS_DIR + "/api/files/b.yang"

        try:
            # Setup
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)

            # Tests
            module = ctx.parse_module_path(yin_file, ly.LYS_IN_YIN)
            self.assertIsNotNone(module)
            self.assertEqual("a", module.name())

            module = ctx.parse_module_path(yang_file, ly.LYS_IN_YANG)
            self.assertIsNotNone(module)
            self.assertEqual("b", module.name())

        except Exception as e:
            self.fail(e)

    def test_ly_ctx_parse_module_path_invalid(self):
        yang_folder = config.TESTS_DIR + "/api/files"
        yin_file = config.TESTS_DIR + "/api/files/a.yin"

        try:
            # Setup
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)

            # Tests
            # parsing with wrong format should raise runtime exception
            module = ctx.parse_module_path(yin_file, ly.LYS_IN_YANG)
            raise UnexpectedError("Exception not thrown")

        except UnexpectedError as e:
            self.fail(e)

        except RuntimeError as e:
            return

        except Exception as e:
            self.fail(e)

    def test_ly_module_print_mem_tree(self):
        yang_folder = config.TESTS_DIR + "/api/files"

        try:
            # Setup
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)
            module = ctx.parse_module_mem(lys_module_a, ly.LYS_IN_YIN)
            self.assertIsNotNone(module)

            # Tests
            result = module.print_mem(ly.LYS_OUT_TREE, 0)
            self.assertEqual(result_tree, result)

        except Exception as e:
            self.fail(e)

    def test_ly_module_print_mem_yang(self):
        yang_folder = config.TESTS_DIR + "/api/files"

        try:
            # Setup
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)
            module = ctx.parse_module_mem(lys_module_a, ly.LYS_IN_YIN)
            self.assertIsNotNone(module)

            # Tests
            result = module.print_mem(ly.LYS_OUT_YANG, 0)
            self.assertEqual(result_yang, result)

        except Exception as e:
            self.fail(e)

    def test_ly_module_print_mem_yin(self):
        yang_folder = config.TESTS_DIR + "/api/files"

        try:
            # Setup
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)
            module = ctx.parse_module_mem(lys_module_a, ly.LYS_IN_YIN)
            self.assertIsNotNone(module)

            # Tests
            result = module.print_mem(ly.LYS_OUT_YIN, 0)
            self.assertEqual(result_yin, result)

        except Exception as e:
            self.fail(e)

    def test_ly_schema_node_find_path(self):
        yang_folder = config.TESTS_DIR + "/api/files"

        try:
            # Setup
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)
            module = ctx.parse_module_mem(lys_module_a, ly.LYS_IN_YIN)
            self.assertIsNotNone(module)
            schema_node = module.data()
            self.assertIsNotNone(schema_node)

            # Tests
            set = schema_node.find_path("/a:x/*")
            self.assertIsNotNone(set)
            self.assertEqual(5, set.number())
            set = schema_node.find_path("/a:x//*")
            self.assertIsNotNone(set)
            self.assertEqual(6, set.number())
            set = schema_node.find_path("/a:x//.")
            self.assertIsNotNone(set)
            self.assertEqual(7, set.number())

        except Exception as e:
            self.fail(e)

    def test_ly_schema_node_path(self):
        yang_folder = config.TESTS_DIR + "/api/files"

        try:
            # Setup
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)
            module = ctx.parse_module_mem(lys_module_a, ly.LYS_IN_YIN)
            self.assertIsNotNone(module)
            schema_node = module.data()
            self.assertIsNotNone(schema_node)

            # Tests
            template = "/a:x/a:bar-gggg"
            set = schema_node.find_path(template)
            self.assertIsNotNone(set)
            schema = set.schema()[0]
            path = schema.path(0)
            self.assertEqual(template, path)

        except Exception as e:
            self.fail(e)

    def test_ly_module_data_instatiables(self):
        yang_folder = config.TESTS_DIR + "/api/files"
        module_name = "b"

        try:
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)

            module = ctx.load_module(module_name)
            self.assertIsNotNone(module)
            self.assertEqual(module_name, module.name())

            instantiables = module.data_instantiables(0)
            self.assertIsNotNone(instantiables)
            self.assertEqual(1, len(instantiables))

        except Exception as e:
            self.fail(e)

    def test_ly_schema_child_instatiables(self):
        yang_folder = config.TESTS_DIR + "/api/files"
        module_name = "b"

        try:
            ctx = ly.Context(yang_folder)
            self.assertIsNotNone(ctx)

            module = ctx.load_module(module_name)
            self.assertIsNotNone(module)
            self.assertEqual(module_name, module.name())

            instantiables = module.data_instantiables(0)
            self.assertIsNotNone(instantiables)
            self.assertEqual(1, len(instantiables))
            child_instantiables = instantiables[0].child_instantiables(0)
            self.assertIsNotNone(child_instantiables)
            self.assertEqual(3, len(child_instantiables))

        except Exception as e:
            self.fail(e)

if __name__ == '__main__':
    unittest.main()
